spool MunAlg
/* 

Autor: Lic. Guillermo Cherencio

Implementacion DER
Programacion Aplicacion
Universidad Nacional de Lujan

Forma de uso:

SQL> start <path>\MunAlg.sql   (antes ejecutar Mundial.sql,MunDat.sql)

Algunos Queries sobre
la Implementacion del enunciado DER del Mundial de Futbol
Ejercicio Nro. 10
Comparaciones entre SQL y Algebra Relacional

Formato Gral. de este script:

<< SETEOS DE AMBIENTE/PARAMETROS REQUERIDOS PARA LA EJECUCION >>

<< BORRO PROCEDURES/FUNCIONES >>

<< EJECUTO QUERIES/CREO PROCEDURES >>


En MunQry.lst queda la salida de este script.

*/

/* Sets para la sesion actual */
clear buffer
set serveroutput on
set verify off
set termout on
ALTER SESSION SET NLS_DATE_FORMAT = 'dd/mm/yyyy'
/

DROP PROCEDURE PAR_POSIBLE
/

/*
Cuales son los jugadores de Argentina?

Algebra:
SELECT (JUGADOR: NOMBRE_EQ = 'ARGENTINA') [NOMBRE]

Implica 2 operaciones: 
1) Seleccion : de todos los jugadores de Argentina
2) Proyeccion: de todas las columnas que posee la tabla jugador solo me interesa el Nombre
*/

SELECT JG.NOMBRE FROM JUGADOR JG
WHERE NOMBRE_EQ = 'ARGENTINA' 
/


/*
Cuales son los jugadores de Argentina y de Brasil?

En vez de utilizar una seleccion NOMBRE_EQ = 'ARGENTINA' OR NOMBRE_EQ = 'BRASIL' resuelvo
esta consulta utilizando una UNION entre los jugadores de Argentina y los de Brasil

Algebra:
JUGARG := SELECT (JUGADOR: NOMBRE_EQ = 'ARGENTINA') [NOMBRE]
JUGBRA := SELECT (JUGADOR: NOMBRE_EQ = 'BRASIL') [NOMBRE]
SOLUCION := JUGARG U JUGBRA

Implica 2 operaciones: 
1) Seleccion : de todos los jugadores de Argentina
2) Proyeccion: de todas las columnas que posee la tabla jugador solo me interesa el Nombre
3) Guardar el resultado de esta operacion
4) Seleccion : de todos los jugadores de Brasil
5) Proyeccion: de todas las columnas que posee la tabla jugador solo me interesa el Nombre
6) Guardar el resultado de esta operacion
7) Hacer la Union de las dos relaciones guardadas

*/
SELECT JG.NOMBRE FROM JUGADOR JG
WHERE NOMBRE_EQ = 'ARGENTINA' 
UNION
SELECT JG.NOMBRE FROM JUGADOR JG
WHERE NOMBRE_EQ = 'BRASIL' 
/

/*
Otras soluciones al problema anterior

Algebra:
SELECT (JUGADOR: NOMBRE_EQ = 'ARGENTINA' OR NOMBRE_EQ = 'BRASIL') [NOMBRE]

*/
SELECT JG.NOMBRE FROM JUGADOR JG
WHERE NOMBRE_EQ = 'ARGENTINA' OR NOMBRE_EQ = 'BRASIL'
/

/*
Otras soluciones al problema anterior
*/
SELECT JG.NOMBRE FROM JUGADOR JG
WHERE NOMBRE_EQ IN('ARGENTINA','BRASIL')
/

/*
Otras soluciones al problema anterior
*/
SELECT JG.NOMBRE FROM JUGADOR JG
WHERE NOMBRE_EQ IN (SELECT NOMBRE FROM EQUIPO WHERE NOMBRE IN('ARGENTINA','BRASIL'))
/

/* 
Cuales son los jugadores que todavia no jugaron?

Logica:
   Todos los jugadores menos los jugadores que ya jugaron

Algebra:
JUGJUG := PAR_JUG [NOMBRE]
JUG    := JUGADOR [NOMBRE]
SOLUCION := JUG - JUGJUG

*/

SELECT JG.NOMBRE FROM JUGADOR JG
MINUS
SELECT PJ.NOMBRE FROM PAR_JUG PJ
/


/*
Otra solucion al problema anterior
Realizar un outer join (left join) entre jugador y par_jug y verificar
cuando el SGBD deba aadir nulos en la columna nombre de par_jug.nombre

Algebra:
JUGNOJUG := OUTERJOIN(JUGADOR,PAR_JUG)
SOLUCION := SELECT (JUGNOJUG: PAR_JUG.NOMBRE = NULL) [JUGADOR.NOMBRE]

*/
SELECT JG.NOMBRE FROM JUGADOR JG, PAR_JUG PJ
WHERE PJ.NOMBRE (+) = JG.NOMBRE AND PJ.NOMBRE IS NULL
/

/* 
Cuales son todos los posibles partidos que se pueden realizar?
Se toman como validos tambien las revanchas, es decir, ARG - BRA
es un partido y BRA - ARG es otro partido

Logica:
   Combino todos los equipos contra todos los equipos y de ese grupo
"de todos contra todos" elimino los que sean contra si mismos, ya que
un equipo no puede jugar contra si mismo.

Algebra:
EQUIPO1 := EQUIPO [NOMBRE]
EQUIPO2 := EQUIPO [NOMBRE]
EQ1xEQ2 := EQUIPO1 * EQUIPO2
SOLUCION:= SELECT (EQ1xEQ2: EQUIPO1.NOMBRE <> EQUIPO2.NOMBRE)
*/
SELECT EQ1.NOMBRE, EQ2.NOMBRE FROM EQUIPO EQ1,EQUIPO EQ2
WHERE EQ1.NOMBRE <> EQ2.NOMBRE
/

/* 
Cuales son todos los posibles partidos que se pueden realizar,
 teniendo en cuenta los partidos que ya se hicieron?
Se toman como validos tambien las revanchas, es decir, ARG - BRA
es un partido y BRA - ARG es otro partido

Logica:
   Combino todos los equipos contra todos los equipos y de ese grupo
"de todos contra todos" elimino los que sean contra si mismos, ya que
un equipo no puede jugar contra si mismo.  De esta manera tengo todos
los partidos posibles, a esta relacion le resto los partidos ya hechos
teniendo en cuenta la combinacion equipo1 vs equipo2 y equipo2 vs equipo1

Algebra:
EQUIPO1 := EQUIPO [NOMBRE]
EQUIPO2 := EQUIPO [NOMBRE]
EQ1xEQ2 := EQUIPO1 * EQUIPO2
POSIBLES:= SELECT (EQ1xEQ2: EQUIPO1.NOMBRE <> EQUIPO2.NOMBRE)
HECHOS1 := PARTIDO [NOMBRE_EQ1,NOMBRE_EQ2]
HECHOS2 := PARTIDO [NOMBRE_EQ2,NOMBRE_EQ1]
HECHOS  := HECHOS1 U HECHOS2
SOLUCION:= POSIBLES - HECHOS
*/

SELECT EQ1.NOMBRE "NOMBRE_EQ1", EQ2.NOMBRE "NOMBRE_EQ2" FROM EQUIPO EQ1,EQUIPO EQ2
WHERE EQ1.NOMBRE <> EQ2.NOMBRE
MINUS
SELECT HECHOS.NOMBRE_EQ1,HECHOS.NOMBRE_EQ2
FROM (
SELECT NOMBRE_EQ1,NOMBRE_EQ2 FROM PARTIDO
UNION
SELECT NOMBRE_EQ2 "NOMBRE_EQ1",NOMBRE_EQ1 "NOMBRE_EQ2" FROM PARTIDO
) HECHOS
/

/* 
Cuales son todos los posibles partidos que se pueden realizar?

No se toman como validos las revanchas, es decir, ARG - BRA
es un partido y BRA - ARG es el mismo partido.  Utilizo uno u el
otro, pero no ambos

Logica:
   Combino todos los equipos contra todos los equipos y de ese grupo
"de todos contra todos" elimino los que sean contra si mismos, ya que
un equipo no puede jugar contra si mismo.  De este grupo verifico que
no pueda darse la situacion descripta anteriormente, voy a solucionar 
utilizando codigo PL/SQL, haciendo un procedimiento que declare un 
cursor con la solucion anterior, lo cargue en vectores y por ultimo 
que recorra dichos vectores para no imprimir los casos no deseados.

Algebra:
?
sql:
?
*/

CREATE OR REPLACE PROCEDURE PAR_POSIBLE
AS
   -- Declaro Cursor
   CURSOR c_PosiblesEquipos IS
          SELECT EQ1.NOMBRE "EQUIPO1", EQ2.NOMBRE "EQUIPO2"
          FROM EQUIPO EQ1,EQUIPO EQ2
          WHERE EQ1.NOMBRE <> EQ2.NOMBRE;
   -- Declaro Tablas a utilizar como si fueran tipos de datos
   TYPE tbl_Equipo1 IS TABLE OF EQUIPO.NOMBRE%TYPE
      INDEX BY BINARY_INTEGER;
   TYPE tbl_Equipo2 IS TABLE OF EQUIPO.NOMBRE%TYPE
      INDEX BY BINARY_INTEGER;
   TYPE tbl_Imprimo IS TABLE OF BOOLEAN
      INDEX BY BINARY_INTEGER;
   -- Declaro variables de estos tipos de datos vector
   v_equipo1 tbl_Equipo1;
   v_equipo2 tbl_Equipo2;
   v_imprimo tbl_Imprimo;
   -- Declaro otras variables para el manejo de subindices
   i INTEGER:=1;
   n INTEGER;
   w INTEGER;
   -- Declaro variable para leer cursor hasta "EOF"
   lhayPosiblesEquipos BOOLEAN;
BEGIN
   -- Abro cursor
   OPEN c_PosiblesEquipos;
/*
   Leo cursor, cargo vectores con datos del cursor
   y en un principio asumo que todos estos partidos son
   validos 
*/
   lhayPosiblesEquipos := TRUE;
   WHILE lhayPosiblesEquipos LOOP
      FETCH c_PosiblesEquipos INTO v_equipo1(i), v_equipo2(i);
      lhayPosiblesEquipos := c_PosiblesEquipos%FOUND;
      IF c_PosiblesEquipos%FOUND THEN
         v_imprimo(i) := TRUE;
      END IF;
      i := i + 1;
   END LOOP;
   -- Cierro Cursor
   CLOSE c_PosiblesEquipos;
   -- ojo! con el valor de i: uno por incremento y otro por %FOUND
/*
   Leo cada partido y busco en los restantes partidos (excepto contra
   si mismo) la posibilidad de que este partido(n) tenga un equivalen
   te(w), en tal caso (w) no debera imprimirse o tomarse como un par-
   tido valido.
*/
   DBMS_OUTPUT.PUT_LINE('Partidos Validos:');
   DBMS_OUTPUT.PUT_LINE('-----------------');
   FOR n IN 1..(i-2) LOOP
      IF v_imprimo(n) THEN
         FOR w IN 1..(i-2) LOOP
            IF w <> n THEN
               IF v_equipo1(n) = v_equipo2(w) AND 
                  v_equipo2(n) = v_equipo1(w) THEN
                  v_imprimo(w) := FALSE;
               END IF;
            END IF;
         END LOOP;
         DBMS_OUTPUT.PUT_LINE(v_equipo1(n) || ' vs ' || v_equipo2(n));
      END IF;     
   END LOOP;
/*
   Imprimo todos los partidos que descarte en el proceso anterior
*/
   DBMS_OUTPUT.PUT_LINE('.');
   DBMS_OUTPUT.PUT_LINE('Partidos No Validos o Revanchas:');
   DBMS_OUTPUT.PUT_LINE('--------------------------------');
   FOR n IN 1..(i-2) LOOP
      IF NOT v_imprimo(n) THEN
         DBMS_OUTPUT.PUT_LINE(v_equipo1(n) || ' vs ' || v_equipo2(n));
      END IF;     
   END LOOP;
END;
/

/* 
Cuales son todos los posibles partidos que se pueden realizar?

No se toman como validos las revanchas, es decir, ARG - BRA
es un partido y BRA - ARG es el mismo partido.  Utilizo uno u el
otro, pero no ambos

Logica:
   Combino todos los equipos contra todos los equipos teniendo en
cuenta unicamente los equipos que esten alfabeticamente por encima
del que estoy combinando en este momento.  De esta manera, ya no es
necesario filtrar los partidos contra si mismo, por el hecho de que
ningun equipo esta alfabeticamente por encima de si mismo.

Algebra:
EQUIPO1 := EQUIPO [NOMBRE]
EQUIPO2 := EQUIPO [NOMBRE]
SOLUCION:= SELECT (EQUIPO1 * EQUIPO2: EQUIPO1.NOMBRE > EQUIPO2.NOMBRE)

*/
SELECT EQ1.NOMBRE, EQ2.NOMBRE FROM EQUIPO EQ1,EQUIPO EQ2
WHERE EQ1.NOMBRE > EQ2.NOMBRE
/

/* 
Cuales son todos los posibles partidos que se pueden realizar?

No se toman como validos las revanchas, es decir, ARG - BRA
es un partido y BRA - ARG es el mismo partido.  Utilizo uno u el
otro, pero no ambos

Logica:
   Combino todos los equipos contra todos los equipos teniendo en
cuenta unicamente los equipos que esten alfabeticamente por debajo
del que estoy combinando en este momento.  De esta manera, ya no es
necesario filtrar los partidos contra si mismo, por el hecho de que
ningun equipo esta alfabeticamente por debajo de si mismo.

Algebra:
EQUIPO1 := EQUIPO [NOMBRE]
EQUIPO2 := EQUIPO [NOMBRE]
SOLUCION:= SELECT (EQUIPO1 * EQUIPO2: EQUIPO1.NOMBRE > EQUIPO2.NOMBRE)

*/
SELECT EQ1.NOMBRE, EQ2.NOMBRE FROM EQUIPO EQ1,EQUIPO EQ2
WHERE EQ1.NOMBRE < EQ2.NOMBRE
/


/* 
Cuales son todos los posibles partidos que se pueden realizar,
 teniendo en cuenta los partidos que ya se hicieron?

No se toman como validos las revanchas, es decir, ARG - BRA
es un partido y BRA - ARG es el mismo partido.  Utilizo uno u el
otro, pero no ambos

Logica:
   Combino todos los equipos contra todos los equipos teniendo en
cuenta unicamente los equipos que esten alfabeticamente por encima
del que estoy combinando en este momento.  De esta manera, ya no es
necesario filtrar los partidos contra si mismo, por el hecho de que
ningun equipo esta alfabeticamente por encima de si mismo.
Asi obtengo todos los partidos posibles, a esta relacion le resto 
los partidos ya hechos teniendo en cuenta la combinacion 
equipo1 vs equipo2 y equipo2 vs equipo1

Algebra:
EQUIPO1 := EQUIPO [NOMBRE]
EQUIPO2 := EQUIPO [NOMBRE]
POSIBLES:= SELECT (EQUIPO1 * EQUIPO2: EQUIPO1.NOMBRE > EQUIPO2.NOMBRE)
HECHOS1 := PARTIDO [NOMBRE_EQ1,NOMBRE_EQ2]
HECHOS2 := PARTIDO [NOMBRE_EQ2,NOMBRE_EQ1]
HECHOS  := HECHOS1 U HECHOS2
SOLUCION:= POSIBLES - HECHOS

*/
SELECT EQ1.NOMBRE, EQ2.NOMBRE FROM EQUIPO EQ1,EQUIPO EQ2
WHERE EQ1.NOMBRE > EQ2.NOMBRE
MINUS
SELECT HECHOS.NOMBRE_EQ1,HECHOS.NOMBRE_EQ2
FROM (
SELECT NOMBRE_EQ1,NOMBRE_EQ2 FROM PARTIDO
UNION
SELECT NOMBRE_EQ2 "NOMBRE_EQ1",NOMBRE_EQ1 "NOMBRE_EQ2" FROM PARTIDO
) HECHOS
/


/* 
Quien o Quienes son los arbitros o jueces de linea que dirigieron/controlaron
 a todos los equipos?

Logica:
   Armar 2 relaciones: una con la lista de todos los equipos llamemosla tabeq 
y otra con las autoridades (arbitros + jueces) y equipos dirigidos/controlados
llamemosla auteq.  Por ultimo la respuesta a esta pregunta es: auteq / tabeq

Algebra:
TABEQ    := EQUIPO [NOMBRE]
AREQ1    := PARTIDO [IDARBITRO,NOMBRE_EQ1]
AREQ2    := PARTIDO [IDARBITRO,NOMBRE_EQ2]
JL1EQ1   := PARTIDO [IDLINEA1,NOMBRE_EQ1]
JL1EQ2   := PARTIDO [IDLINEA1,NOMBRE_EQ2]
JL2EQ1   := PARTIDO [IDLINEA2,NOMBRE_EQ1]
JL2EQ2   := PARTIDO [IDLINEA2,NOMBRE_EQ2]
AUTEQ    := AREQ1 U AREQ2 U JL1EQ1 U JL1EQ2 U JL2EQ1 U JL2EQ2
SOLUCION1:= AUTEQ / TABEQ
SOLUCION := JOIN(SOLUCION1,AUTORIDAD) [AUTORIDAD.NOMBRE]

Sql:
El lenguaje no cuenta con una operacion de DIVISION al igual que el algebra,
asi que tendremos que utilizar los elementos existes para implementar una
division, seguramente podremos encontrar varias formas de realizarlo.  Una
forma podria ser: Construimos a tabla a ser dividida y por cada fila de la
misma, verificamos que la resta de la tabla divisora (tabla equipos) menos
los correspondientes a la fila actual (lista de equipos dirigidos/controla-
dos por la autoridad actual) me de vacio.  El hecho de dar vacia la resta,
implicaria que ambos conjuntos eran iguales, tenian los mismos elementos que,
al ser restados me da como resultado un conjunto vacio.
En el ejemplo de la pag. 194 del apunte de Algebra/Calculo Relacional, pueden
observar que la tabla a ser dividida tiene 2 columnas mientras que la tabla
divisora tiene solo una, esto no es una condicion "sin equa non" para sql, ya
que no implementa la operacion de division y la misma la construimos a traves
del razonamiento anterior.

Logica aplicada en sql:
-Obtengo todos los ID's de las autoridades que participaron en la direccion
 de los distintos partidos
-Por cada uno de estos ID's verifico que:
 la resta entre la lista de equipos posibles menos la lista de equipos que este
 ID ha dirigido/controlado sea un conjunto vacio
-Una vez verificado esto, realizo el join natural con la tabla autoridad para
 obtener el nombre del arbitro/juez de linea.   
*/

SELECT AUT.NOMBRE FROM (
SELECT PAR.IDARBITRO "ID" FROM PARTIDO PAR
UNION
SELECT PAR.IDLINEA1  "ID" FROM PARTIDO PAR
UNION
SELECT PAR.IDLINEA2  "ID" FROM PARTIDO PAR
) AUTEQ, AUTORIDAD AUT
WHERE NOT EXISTS(
                 SELECT NOMBRE "EQUIPO" FROM EQUIPO
                 MINUS (
                      SELECT PAR.NOMBRE_EQ1 "EQUIPO" FROM PARTIDO PAR
                      WHERE  PAR.IDARBITRO = AUTEQ.ID
                      UNION
                      SELECT PAR.NOMBRE_EQ2 "EQUIPO" FROM PARTIDO PAR
                      WHERE  PAR.IDARBITRO = AUTEQ.ID
                      UNION
                      SELECT PAR.NOMBRE_EQ1 "EQUIPO" FROM PARTIDO PAR
                      WHERE  PAR.IDLINEA1 = AUTEQ.ID
                      UNION
                      SELECT PAR.NOMBRE_EQ2 "EQUIPO" FROM PARTIDO PAR
                      WHERE  PAR.IDLINEA1 = AUTEQ.ID
                      UNION
                      SELECT PAR.NOMBRE_EQ1 "EQUIPO" FROM PARTIDO PAR
                      WHERE  PAR.IDLINEA2 = AUTEQ.ID
                      UNION
                      SELECT PAR.NOMBRE_EQ2 "EQUIPO" FROM PARTIDO PAR
                      WHERE  PAR.IDLINEA2 = AUTEQ.ID
                       )
                ) AND  AUTEQ.ID = AUT.IDAUT
/

/*

Idem anterior aplicando la siguiente logica:

La cantidad de filas devueltas del join natural entre cada uno de los ID's de autoridades
en cuestion y la tabla de equipos debera ser igual a la cantidad de filas de la tabla e-
quipos; es decir, que la resta entre estos dos conjuntos debera dar un conjunto vacion en
caso de que ambos sean iguales

Previamente, creo una vista que sera luego utilizada en la consulta, esto me permite bajar
el nivel de complejidad de las consultas, al tiempo que gano en claridad
*/
CREATE OR REPLACE VIEW AUT_EQ
AS
   SELECT PAR.IDARBITRO "AUTORIDAD", PAR.NOMBRE_EQ1 "EQUIPO" 
   FROM PARTIDO PAR 
   UNION
   SELECT PAR.IDARBITRO "AUTORIDAD", PAR.NOMBRE_EQ2 "EQUIPO"
   FROM PARTIDO PAR
   UNION
   SELECT PAR.IDLINEA1  "AUTORIDAD", PAR.NOMBRE_EQ1 "EQUIPO" 
   FROM PARTIDO PAR
   UNION
   SELECT PAR.IDLINEA1  "AUTORIDAD", PAR.NOMBRE_EQ2 "EQUIPO" 
   FROM PARTIDO PAR
   UNION
   SELECT PAR.IDLINEA2  "AUTORIDAD", PAR.NOMBRE_EQ1 "EQUIPO" 
   FROM PARTIDO PAR
   UNION
   SELECT PAR.IDLINEA2  "AUTORIDAD", PAR.NOMBRE_EQ2 "EQUIPO" 
   FROM PARTIDO PAR
/


SELECT AUT.NOMBRE FROM (
SELECT PAR.IDARBITRO "ID" FROM PARTIDO PAR
UNION
SELECT PAR.IDLINEA1  "ID" FROM PARTIDO PAR
UNION
SELECT PAR.IDLINEA2  "ID" FROM PARTIDO PAR
) AUTEQ, AUTORIDAD AUT
WHERE NOT EXISTS 
    ( SELECT COUNT(*) FROM AUT_EQ, EQUIPO EQ
      WHERE AUT_EQ.AUTORIDAD = AUTEQ.ID AND AUT_EQ.EQUIPO = EQ.NOMBRE
      MINUS
      SELECT COUNT(*) FROM EQUIPO
      ) AND   AUTEQ.ID = AUT.IDAUT
/
