spool MunQry
/* 

Autor: Lic. Guillermo Cherencio

Implementacion DER
Programacion Aplicacion
Universidad Nacional de Lujan

Forma de uso:

SQL> start <path>\MunQry.sql   (antes ejecutar Mundial.sql,MunDat.sql)

Algunos Queries y procesos tipicos sobre
la Implementacion del enunciado DER del Mundial de Futbol
Ejercicio Nro. 10

Formato Gral. de este script:

<< SETEOS DE AMBIENTE/PARAMETROS REQUERIDOS PARA LA EJECUCION >>

<< BORRO PROCEDURES >>

<< EJECUTO QUERIES/CREO PROCEDURES >>


En MunQry.lst queda la salida de este script.

*/

/* Sets para la sesion actual */
clear buffer
set serveroutput on
set verify off
set termout on
ALTER SESSION SET NLS_DATE_FORMAT = 'dd/mm/yyyy'
/

DROP PROCEDURE INS_PAR_JUG
/

/*
Cuales son los jugadores de Argentina que hasta ahora no han jugado?
*/

SELECT JG.NOMBRE FROM JUGADOR JG
WHERE NOMBRE_EQ = 'ARGENTINA' AND NOT EXISTS(SELECT * FROM PAR_JUG PJ
                                             WHERE PJ.NOMBRE = JG.NOMBRE)
/

/*
Cuales son los puestos aun no cubiertos por jugadores argentinos en el partido
entre Argentina y Brasil del 18-06-2001?

Todos los puestos posibles menos los puestos ocupados en ese partido
*/

SELECT DESCRIPCION FROM PUESTO
MINUS
SELECT PT.DESCRIPCION FROM PUESTO PT, PAR_JUG PJ, JUGADOR JG
WHERE PJ.FECHA      = TO_DATE('18-06-2001') AND 
     (PJ.NOMBRE_EQ1 = 'ARGENTINA' OR  PJ.NOMBRE_EQ1 = 'BRASIL')   AND 
      PJ.NOMBRE     = JG.NOMBRE   AND JG.NOMBRE_EQ  = 'ARGENTINA' AND 
      PJ.IDPUE      = PT.IDPUE
/


/*
Cuantos puntos tiene Argentina, teniendo en cuenta los partidos jugados y 
sabiendo que son 3 puntos por ganar,1 punto por empate y 0 por perder?

La cantidad de partidos ganados por 3 mas La cantidad de partidos empatados
junto los dos grupos y hago una sumatoria de los puntos obtenidos
*/

SELECT SUM(PUNTOS) "TOTAL PUNTOS" FROM (
   SELECT (COUNT(*)*3) "PUNTOS" FROM PARTIDOV PV        --partidos ganados
   WHERE (PV.NOMBRE_EQ1 = 'ARGENTINA' AND PV.RESULTADO = 'L') OR 
         (PV.NOMBRE_EQ2 = 'ARGENTINA' AND PV.RESULTADO = 'V')
   UNION 
   SELECT COUNT(*) "PUNTOS" FROM PARTIDOV PV           --partidos empatados
   WHERE (PV.NOMBRE_EQ1 = 'ARGENTINA' OR PV.NOMBRE_EQ2 = 'ARGENTINA') AND
          PV.RESULTADO = 'E'
   UNION
   SELECT 0 "PUNTOS" FROM DUAL                         --partidos perdidos
)
/



/* 

Emita un listado con la tabla de posiciones, obviamente, comenzando por
los equipos que mas puntos tienen

Agrupar por equipo1 y sumar puntos de todos los partidos locales
Agrupar por equipo2 y sumar puntos de todos los partidos visitantes
(Ojo con los empates porque es un punto para cada equipo)
Agrupar por equipo1 y sumar puntos de todos los partidos empates
Agrupar por equipo2 y sumar puntos de todos los partidos empates

Agrupar tambien los perdedores para indicar puntuacion = 0
*/

SELECT EQUIPO "EQUIPO",SUM(PUNTOS) "TOTAL PUNTOS" FROM (
SELECT PV.NOMBRE_EQ1 "EQUIPO",(COUNT(PV.FECHA)*3) "PUNTOS" FROM PARTIDOV PV
WHERE PV.RESULTADO = 'L'
GROUP BY PV.NOMBRE_EQ1
UNION
SELECT PV.NOMBRE_EQ2 "EQUIPO",(COUNT(PV.FECHA)*3) "PUNTOS" FROM PARTIDOV PV
WHERE PV.RESULTADO = 'V'
GROUP BY PV.NOMBRE_EQ2
UNION
SELECT PV.NOMBRE_EQ1 "EQUIPO",COUNT(PV.FECHA) "PUNTOS" FROM PARTIDOV PV
WHERE PV.RESULTADO = 'E'
GROUP BY PV.NOMBRE_EQ1
UNION
SELECT PV.NOMBRE_EQ2 "EQUIPO",COUNT(PV.FECHA) "PUNTOS" FROM PARTIDOV PV
WHERE PV.RESULTADO = 'E'
GROUP BY PV.NOMBRE_EQ2
UNION
SELECT PV.NOMBRE_EQ1 "EQUIPO",0 "PUNTOS" FROM PARTIDOV PV
WHERE PV.RESULTADO = 'V'
GROUP BY PV.NOMBRE_EQ1
UNION
SELECT PV.NOMBRE_EQ2 "EQUIPO",0 "PUNTOS" FROM PARTIDOV PV
WHERE PV.RESULTADO = 'L'
GROUP BY PV.NOMBRE_EQ2
)
GROUP BY EQUIPO
ORDER BY SUM(PUNTOS) DESC
/


/*

Es normal que un equipo respete el 80% o 90% de su formacion de un partido a
otro, por lo tanto, para facilitar la asignacion de jugadores a partidos, se
requiere de un "copiador" que "copie" la formacion que tuvo un equipo en un
partido determinado a otro nuevo partido.  Luego de usar este "copiador", los
operadores del sistema realizaran los ajustes necesarios sobre la formacion,
pero ya tendran, en promedio el 80% de su trabajo hecho.
Implementar una o mas soluciones posibles a este problema.

Llamemos "partido origen" al partido de donde voy a copiar la formacion de un
equipo y "partido destino" al partido nuevo en donde voy a copiar la formacion
de un equipo.

Solucion 1:
Puedo implementar uno o mas queries de actualizacion.

Solucion 2:
Puedo implementar un procedimiento al cual le indico "partido origen" y "parti-
do destino" y realiza todo el proceso.

Cada solucion tiene sus ventajas y desventajas.  Discutirlas en clase.


Para probar este ejercicio agrego un nuevo equipo y partido 
partido origen=Argentina respeta la misma formacion que tuvo contra Brasil 
               el 18-06-2001
partido destino=Argentina Vs Rusia el 22-06-2001

*/
INSERT INTO EQUIPO(NOMBRE) VALUES('RUSIA');
INSERT INTO PARTIDO VALUES(TO_DATE('22-06-2001'),'ARGENTINA','RUSIA','20:00','22:00',1,2,3);

/* 
Solucion 1: Uno o mas queries de actualizacion 
Me aseguro de que no exista formacion en partido destino 
y luego inserto la informacion desde partido origen
*/
DELETE FROM PAR_JUG PJ WHERE PJ.FECHA      = TO_DATE('22-06-2001') AND 
                             PJ.NOMBRE_EQ1 = 'ARGENTINA'
/
INSERT INTO PAR_JUG(FECHA,NOMBRE_EQ1,NOMBRE,IDPUE,GOLES)
SELECT TO_DATE('22-06-2001'),'ARGENTINA',PJ.NOMBRE,PJ.IDPUE,0
FROM PAR_JUG PJ,JUGADOR JG
WHERE PJ.FECHA  = TO_DATE('18-06-2001') AND PJ.NOMBRE_EQ1 = 'ARGENTINA' AND
      PJ.NOMBRE = JG.NOMBRE AND JG.NOMBRE_EQ = 'ARGENTINA'
/


/* 
Solucion 2: Creacion de un procedimiento
Me aseguro de que no exista formacion en partido destino 
y luego inserto la informacion desde partido origen
*/
CREATE OR REPLACE PROCEDURE INS_PAR_JUG(OFECHA IN DATE,ONOMBRE_EQ1 IN VARCHAR2,
                                        DFECHA IN DATE,DNOMBRE_EQ1 IN VARCHAR2,
                                        PNOMBRE_EQ IN VARCHAR2)
AS
   n_jug	NUMBER := 0;
BEGIN
   /* Verfico que la formacion no este previamente ingresada */
   SELECT COUNT(*) INTO n_jug FROM PAR_JUG PJ, JUGADOR JG
   WHERE PJ.FECHA = DFECHA AND PJ.NOMBRE_EQ1 = DNOMBRE_EQ1 AND PJ.NOMBRE = JG.NOMBRE AND 
         JG.NOMBRE_EQ = PNOMBRE_EQ;
   IF n_jug > 0 THEN
      /* Si la formacion estuvo previamente ingresada, la borro e ingreso la nueva */
      DELETE FROM PAR_JUG PJ WHERE PJ.FECHA      = DFECHA AND
                                   PJ.NOMBRE_EQ1 = DNOMBRE_EQ1 AND
                                   PJ.NOMBRE IN ( SELECT JG.NOMBRE FROM JUGADOR JG
                                                  WHERE JG.NOMBRE_EQ = PNOMBRE_EQ );
      /* DBMS_OUTPUT.PUT_LINE('Borre ' || TO_CHAR(n_jug) ); */
   END IF;
   /* Ingreso la formacion del equipo PNOMBRE_EQ al partido DFECHA DNOMBRE_EQ1 */
   INSERT INTO PAR_JUG(FECHA,NOMBRE_EQ1,NOMBRE,IDPUE,GOLES)
   SELECT DFECHA,DNOMBRE_EQ1,PJ.NOMBRE,PJ.IDPUE,0
   FROM PAR_JUG PJ,JUGADOR JG
   WHERE PJ.FECHA  = OFECHA    AND PJ.NOMBRE_EQ1 = ONOMBRE_EQ1 AND
         PJ.NOMBRE = JG.NOMBRE AND JG.NOMBRE_EQ = PNOMBRE_EQ;
   /* Doy por cumplida esta transaccion */
   COMMIT;
EXCEPTION
   WHEN OTHERS THEN 
      DBMS_OUTPUT.PUT_LINE('Maldicion Error: ' || SQLERRM);
      ROLLBACK;
END;
/

/* Ejecuto el Store Procedure */
EXEC INS_PAR_JUG(TO_DATE('18-06-2001'),'ARGENTINA',TO_DATE('22-06-2001'),'ARGENTINA','ARGENTINA')
/