spool Mundial
/* 

Autor: Lic. Guillermo Cherencio

Implementacion DER
Programacion Aplicacion
Universidad Nacional de Lujan

Forma de uso:

SQL> start <path>\Mundial.sql

Creo tablas, constraints, indices, etc.
Para la Implementacion del enunciado DER del Mundial de Futbol
Ejercicio Nro. 10

Formato Gral. de este script:

<< SETEOS DE AMBIENTE/PARAMETROS REQUERIDOS PARA LA EJECUCION >>

<< BORRO TABLAS >>

<< CREO TABLAS/CLAVES >>

<< MUESTRO ESTRUCTURA GRAL. DE TABLAS >>

<< IMPLEMENTO CONTROLES DEL SISTEMA >>

En Mundial.lst queda la salida de este script.

*/

/* Sets para la sesion actual */
clear buffer
set serveroutput on
set verify off
set termout on
ALTER SESSION SET NLS_DATE_FORMAT = 'dd/mm/yyyy'
/
/* Ingreso de parametros */
accept p_len_descs number prompt 'Largo Maximo de las Descripciones:'
/

/* Borro estructuras de datos previas */
DROP VIEW PARTIDOV
/
DROP FUNCTION RESULTADO
/
DROP TABLE PAR_JUG
/
DROP TABLE PARTIDO
/
DROP TABLE JUGADOR
/
DROP TABLE PUESTO
/
DROP TABLE EQUIPO
/
DROP TABLE AUTORIDAD
/

clear buffer
COMMIT
/

/* Creo estructuras */
/* 
Creo Tabla Equipo 

(comienzo por las tablas que no tienen dependencia de otra) 
Observar que una columna puede tener mas de un CONSTRAINT, por ej:
uno para definir clave primaria y otro para establecer el dominio de
la columna 
*/
CREATE TABLE EQUIPO (
   NOMBRE VARCHAR2(&p_len_descs) NOT NULL CONSTRAINT pk_equipo PRIMARY KEY
          CONSTRAINT chk_eq_nombre CHECK 
	  (NOMBRE = UPPER(NOMBRE) AND LENGTH(NOMBRE) > 3),
   OK NUMBER(1,0) DEFAULT 0 CONSTRAINT chk_eq_ok CHECK
   (OK = 0 OR OK = 1)
)
/

/* 
Creo Tabla Autoridad 

(comienzo por las tablas que no tienen dependencia de otra) 
*/
CREATE TABLE AUTORIDAD (
   IDAUT NUMBER(3,0) DEFAULT 1 NOT NULL CONSTRAINT chk_aut_idaut CHECK
   (IDAUT >= 1) CONSTRAINT pk_autoridad PRIMARY KEY,
   NOMBRE VARCHAR2(&p_len_descs) NOT NULL 
          CONSTRAINT chk_aut_nombre CHECK 
	  (NOMBRE = UPPER(NOMBRE) AND LENGTH(NOMBRE) >= 2)
)
/

/* 
Creo Tabla Puesto

(comienzo por las tablas que no tienen dependencia de otra) 
*/
CREATE TABLE PUESTO (
   IDPUE NUMBER(2,0) DEFAULT 1 NOT NULL CONSTRAINT chk_pue_idpue CHECK
   (IDPUE >= 1) CONSTRAINT pk_puesto PRIMARY KEY,
   DESCRIPCION VARCHAR2(&p_len_descs) NOT NULL 
          CONSTRAINT chk_pue_descripcion CHECK 
	  (DESCRIPCION = UPPER(DESCRIPCION) AND LENGTH(DESCRIPCION) > 3)
)
/

/* 
Creo Tabla Jugador

Jugador depende de tabla equipo 
NOMBRE_EQ es not null porque es una relacion obligatoria entre Jugador
y Equipo.  Todo jugador pertenece a un equipo. No vuelvo a realizar las
validaciones de nombre de equipo porque ya lo realiza la propia tabla
equipo (Ahora bien, pueden volver a realizarse por una cuestion de per-
formance) 
*/
CREATE TABLE JUGADOR (
   NOMBRE VARCHAR2(&p_len_descs) NOT NULL CONSTRAINT pk_jugador PRIMARY KEY
          CONSTRAINT chk_jug_nombre CHECK 
	  (NOMBRE = UPPER(NOMBRE) AND LENGTH(NOMBRE) >= 2),
   NOMBRE_EQ VARCHAR2(&p_len_descs) NOT NULL CONSTRAINT fk_jug_equipo 
	  REFERENCES EQUIPO(NOMBRE)
)
/

/* 
Creo Tabla Partido

Partido depende de tabla autoridad, equipo
NOMBRE_EQ1 es not null porque es una relacion obligatoria entre Partido
y Equipo. Idem para NOMBRE_EQ2.
Todo partido involucra a dos equipos. No vuelvo a realizar las
validaciones de nombre de equipo porque ya lo realiza la propia tabla
equipo 
*/
CREATE TABLE PARTIDO (
   FECHA DATE DEFAULT SYSDATE NOT NULL,
   NOMBRE_EQ1 VARCHAR2(&p_len_descs) NOT NULL CONSTRAINT fk_par_equipo1
	  REFERENCES EQUIPO(NOMBRE),
   NOMBRE_EQ2 VARCHAR2(&p_len_descs) NOT NULL CONSTRAINT fk_par_equipo2
          REFERENCES EQUIPO(NOMBRE),
   HORA_INICIO CHAR(5) NOT NULL CONSTRAINT chk_par_horai CHECK
	(SUBSTR(HORA_INICIO,1,2) BETWEEN '00' AND '23' AND 
	 SUBSTR(HORA_INICIO,4,2) BETWEEN '00' AND '59' AND 
 	 SUBSTR(HORA_INICIO,3,1) = ':'),
   HORA_FIN    CHAR(5) NOT NULL CONSTRAINT chk_par_horaf CHECK
	(SUBSTR(HORA_FIN,1,2) BETWEEN '00' AND '23' AND 
	 SUBSTR(HORA_FIN,4,2) BETWEEN '00' AND '59' AND 
 	 SUBSTR(HORA_FIN,3,1) = ':'),
   IDLINEA1 NUMBER(3,0) NOT NULL CONSTRAINT fk_par_aut1 
	  REFERENCES AUTORIDAD(IDAUT),
   IDLINEA2 NUMBER(3,0) NOT NULL CONSTRAINT fk_par_aut2
	  REFERENCES AUTORIDAD(IDAUT),
   IDARBITRO NUMBER(3,0) NOT NULL CONSTRAINT fk_par_aut3
	  REFERENCES AUTORIDAD(IDAUT),
   CONSTRAINT pk_partido PRIMARY KEY (FECHA,NOMBRE_EQ1),
   CONSTRAINT chk_partido CHECK (
	(IDLINEA1 <> IDLINEA2 AND IDLINEA1 <> IDARBITRO AND
	IDLINEA2 <> IDARBITRO) AND (NOMBRE_EQ1 <> NOMBRE_EQ2) AND
	(HORA_INICIO < HORA_FIN) )
)
/

/* Declaro Clave Alternativa para tabla Partido FECHA,NOMBRE_EQ2 */
CREATE UNIQUE INDEX idx_partido ON PARTIDO(FECHA,NOMBRE_EQ2)
/

/* Declaro Clave Alternativa para tabla Partido FECHA,IDARBITRO */
CREATE UNIQUE INDEX idx_partido_ar ON PARTIDO(FECHA,IDARBITRO)
/

/* Declaro Clave Alternativa para tabla Partido FECHA,IDLLINEA1 */
CREATE UNIQUE INDEX idx_partido_l1 ON PARTIDO(FECHA,IDLINEA1)
/

/* Declaro Clave Alternativa para tabla Partido FECHA,IDLLINEA2 */
CREATE UNIQUE INDEX idx_partido_l2 ON PARTIDO(FECHA,IDLINEA2)
/

clear buffer

/* 
Creo Tabla Par_Jug

Par_Jug depende de tabla partido, jugador  
*/
CREATE TABLE PAR_JUG (
   FECHA DATE DEFAULT SYSDATE NOT NULL,
   NOMBRE_EQ1 VARCHAR2(&p_len_descs) NOT NULL,
   NOMBRE VARCHAR2(&p_len_descs) NOT NULL CONSTRAINT fk_pj_jugador 
	  REFERENCES JUGADOR(NOMBRE),
   IDPUE NUMBER(2,0) NOT NULL CONSTRAINT fk_pj_idpue
	  REFERENCES PUESTO(IDPUE),
   GOLES NUMBER(2,0) DEFAULT 0 NOT NULL CONSTRAINT chk_pj_goles CHECK 
	( GOLES >= 0 ),
   	CONSTRAINT pk_par_jug PRIMARY KEY (FECHA,NOMBRE_EQ1,NOMBRE),
	CONSTRAINT fk_pj_partido FOREIGN KEY (FECHA,NOMBRE_EQ1)
	REFERENCES PARTIDO(FECHA,NOMBRE_EQ1)
)
/

/* Muestro estructuras creadas */
DESC EQUIPO
/
DESC JUGADOR
/
DESC AUTORIDAD
/
DESC PUESTO
/
DESC PARTIDO
/
DESC PAR_JUG
/

/* 
Controles sobre este modelo:
-Calcular el resultado de un partido en el orden NOMBRE_EQ1 vs NOMBRE_EQ2
-Controlar que FECHA-EQ2 no exista en el indice FECHA-EQ1 y FECHA-EQ1  no
 exista en indice FECHA-EQ2
-Controlar que al asociar jugadores a un partido sean pertenecientes a los
 equipos que juegan dicho partido
-Controlar que al asociar jugadores a un partido no haya repeticiones del
 puesto con el que juegan (dentro del mismo equipo) La tabla puesto contiene
 11 filas, una por cada puesto.
-Solo se pueden asociar jugadores a un equipo siempre y cuando dicho equipo
 tenga el campo OK = 0, una vez que OK = 1, no se podran asociar jugadores
 a dicho equipo
*/

/*
 Controlar que FECHA-EQ2 no exista en el indice FECHA-EQ1 y FECHA-EQ1  no
 exista en indice FECHA-EQ2
Solucion: Cada vez que se actualizan o insertan datos en la tabla PARTIDO
          verifico que no se de esta situacion
*/
CREATE OR REPLACE TRIGGER trg_iupartido BEFORE UPDATE OR INSERT 
ON PARTIDO
FOR EACH ROW
DECLARE
   n_cant_feq1	NUMBER;
   n_cant_feq2  NUMBER;
BEGIN
   SELECT COUNT(*) INTO n_cant_feq1 FROM PARTIDO
   WHERE FECHA = :new.fecha AND NOMBRE_EQ1 = :new.nombre_eq2;
   SELECT COUNT(*) INTO n_cant_feq2 FROM PARTIDO
   WHERE FECHA = :new.fecha AND NOMBRE_EQ2 = :new.nombre_eq1;
   IF n_cant_feq1 > 0 THEN
      raise_application_error(-20100,:new.nombre_eq2 ||
		 ' no puede jugar el ' || to_char(:new.fecha));
   END IF;
   IF n_cant_feq2 > 0 THEN
      raise_application_error(-20101,:new.nombre_eq1 ||
		 ' no puede jugar el ' || to_char(:new.fecha));
   END IF;
END;
/

/*
Controlar que al asociar jugadores a un partido sean pertenecientes a los
equipos que juegan dicho partido
Solucion: Cada vez que se actualizan o insertan datos en la tabla PAR_JUG
          verifico que se de esta situacion
*/
CREATE OR REPLACE TRIGGER trg_iupar_jug BEFORE UPDATE OR INSERT 
ON PAR_JUG
FOR EACH ROW
DECLARE
   nombre_eqj	JUGADOR.NOMBRE_EQ%TYPE;
   nombre_eq2   JUGADOR.NOMBRE_EQ%TYPE;
   n_select     INTEGER := 1;
   n_cant_pue   NUMBER  := 0;
   ljug_eq1     BOOLEAN;
   nombre_pue   PUESTO.DESCRIPCION%TYPE;
BEGIN
   dbms_output.put_line('uno');
   SELECT NOMBRE_EQ INTO nombre_eqj FROM JUGADOR
   WHERE NOMBRE = :new.nombre;
   n_select := 2;
   SELECT NOMBRE_EQ2 INTO nombre_eq2 FROM PARTIDO
   WHERE FECHA = :new.fecha AND NOMBRE_EQ1 = :new.nombre_eq1;
   IF NOT (nombre_eqj = :new.nombre_eq1 OR nombre_eqj = nombre_eq2) THEN
      raise_application_error(-20200,:new.nombre || ' es jugador de ' ||
	nombre_eqj || ' y este partido es ' || :new.nombre_eq1 || ' vs '
	|| nombre_eq2);
   END IF;
   n_select := 3;

   IF (nombre_eqj = :new.nombre_eq1) THEN   --es jugador del equipo1
      SELECT COUNT(*) INTO n_cant_pue FROM PAR_JUG PJ, JUGADOR JG
      WHERE PJ.FECHA  = :new.fecha AND PJ.NOMBRE_EQ1 = :new.nombre_eq1 AND
            JG.NOMBRE = PJ.NOMBRE  AND JG.NOMBRE_EQ  = PJ.NOMBRE_EQ1   AND
            PJ.IDPUE  = :new.idpue;
   ELSE                                     --es jugador del equipo2
      SELECT COUNT(*) INTO n_cant_pue 
      FROM PAR_JUG PJ, JUGADOR JG, PARTIDO PAR
      WHERE PJ.FECHA     = :new.fecha     AND  PJ.NOMBRE_EQ1 = :new.nombre_eq1 AND
            PAR.FECHA    = PJ.FECHA       AND PAR.NOMBRE_EQ1 = PJ.NOMBRE_EQ1   AND
            JG.NOMBRE    = PJ.NOMBRE      AND  JG.NOMBRE_EQ  = PAR.NOMBRE_EQ2  AND
            PJ.IDPUE     = :new.idpue;
   END IF;
   IF (n_cant_pue >= 1) THEN --hay otro jugador en este partido con este mismo puesto
      SELECT DESCRIPCION INTO nombre_pue FROM PUESTO
      WHERE DESCRIPCION = :new.idpue;
      raise_application_error(-20210,:new.nombre || ' no puede jugar de ' ||
	nombre_pue || ' en el partido ' || to_char(:new.fecha) || ' ' ||
        :new.nombre_eq1);
   END IF;

EXCEPTION
   WHEN NO_DATA_FOUND THEN 
      IF n_select = 1 THEN
         raise_application_error(-20500,:new.nombre || ' no existe ' ||
            'en la tabla de jugadores');
      ELSIF n_select = 2 THEN
         raise_application_error(-20600,to_char(:new.fecha) || ' ' ||
            :new.nombre_eq1 || ' no existe en la tabla de partidos');
      ELSE
         raise_application_error(-20610,to_char(:new.idpue) || ' ' ||
            ' no existe en la tabla de puestos');
      END IF;
END;
/
/*
Controlar que
solo se pueden asociar jugadores a un equipo siempre y cuando dicho equipo
tenga el campo OK = 0, una vez que OK = 1, no se podran asociar jugadores
a dicho equipo.
Solucion: Cada vez que se actualizan o insertan datos en la tabla JUGADOR
          verifico que el equipo al que pertenece el jugador tenga el cam-
          po OK = 0
*/
CREATE OR REPLACE TRIGGER trg_iujugador BEFORE UPDATE OR INSERT 
ON JUGADOR
FOR EACH ROW
DECLARE
   n_Ok   EQUIPO.OK%TYPE;
BEGIN
   SELECT OK INTO n_Ok FROM EQUIPO
   WHERE NOMBRE = :new.nombre_eq;
   IF n_Ok <> 0 THEN
      raise_application_error(-20300,:new.nombre || ' no puede ser ' ||
	'jugador del equipo ' || :new.nombre_eq || ' porque la lista ' ||
	' de dicho equipo esta cerrada');
   END IF;
EXCEPTION
   WHEN NO_DATA_FOUND THEN 
	raise_application_error(-20400,:new.nombre_eq || ' no existe ' ||
               ' en la tabla de equipos');
END;
/

/*

Calcular el resultado de un partido en el orden NOMBRE_EQ1 vs NOMBRE_EQ2
Solucion: Se puede crear una View sobre la tabla PARTIDO para generar 3 
          columnas nuevas en base a la informacion pre-existente
          Se requiere la creacion de una funcion a la cual ingresen la
          cantidad de goles de equipo1 y equipo2 y devuelva 'L','E','V'
          local, empate o visitante de equipo1 vs equipo2
          Primero creo la funcion y luego la view que utiliza dicha fun-
          cion.
*/
CREATE OR REPLACE FUNCTION RESULTADO(GOLEQ1 IN NUMBER,GOLEQ2 IN NUMBER)
RETURN CHAR
IS
   c_ret CHAR(1);
BEGIN
   IF GOLEQ1 > GOLEQ2 THEN
      c_ret := 'L';  -- local
   ELSIF GOLEQ1 = GOLEQ2 THEN
      c_ret := 'E';  -- empate
   ELSE
      c_ret := 'V';  -- visitante
   END IF;
   RETURN(c_ret);
END;
/

CREATE OR REPLACE VIEW PARTIDOV(
FECHA,NOMBRE_EQ1,NOMBRE_EQ2,
HORA_INICIO,HORA_FIN,IDLINEA1,IDLINEA2,IDARBITRO,
GOLEQ1,GOLEQ2,RESULTADO)
AS
SELECT P.FECHA,P.NOMBRE_EQ1,P.NOMBRE_EQ2,
       P.HORA_INICIO,P.HORA_FIN,P.IDLINEA1,
       P.IDLINEA2,P.IDARBITRO,
       PEQ1.GOLEQ1,PEQ2.GOLEQ2,
       RESULTADO(PEQ1.GOLEQ1,PEQ2.GOLEQ2)
FROM (
SELECT P.FECHA,P.NOMBRE_EQ1,SUM(PJ.GOLES) GOLEQ1
FROM PARTIDO P, PAR_JUG PJ, JUGADOR JG
WHERE P.FECHA = PJ.FECHA AND P.NOMBRE_EQ1 = PJ.NOMBRE_EQ1 AND
      PJ.NOMBRE = JG.NOMBRE AND JG.NOMBRE_EQ = P.NOMBRE_EQ1
GROUP BY P.FECHA,P.NOMBRE_EQ1 
) PEQ1,(
SELECT P.FECHA,P.NOMBRE_EQ1,SUM(PJ.GOLES) GOLEQ2
FROM PARTIDO P, PAR_JUG PJ, JUGADOR JG
WHERE P.FECHA = PJ.FECHA AND P.NOMBRE_EQ1 = PJ.NOMBRE_EQ1 AND 
      PJ.NOMBRE = JG.NOMBRE AND JG.NOMBRE_EQ = P.NOMBRE_EQ2
GROUP BY P.FECHA,P.NOMBRE_EQ1
) PEQ2, PARTIDO P
WHERE PEQ1.FECHA = PEQ2.FECHA AND PEQ1.NOMBRE_EQ1 = PEQ2.NOMBRE_EQ1
      AND P.FECHA = PEQ1.FECHA AND P.NOMBRE_EQ1 = PEQ1.NOMBRE_EQ1
/

COMMIT
/
clear buffer
spool off
/*************** fin *****************************************************/