spool Mundial2
/* 

Autor: Lic. Guillermo Cherencio

Implementacion DER
Programacion Aplicacion
Universidad Nacional de Lujan

Forma de uso:

SQL> start <path>\Mundial2.sql

Previamente tiene que haber ejecutado el script Mundial

Para la Implementacion del enunciado DER del Mundial de Futbol
Ejercicio Nro. 10
Este script da solucion al problema de Mutating Table/
Table constraining generado por el trigger trg_iupartido ya que
el mismo realiza una consulta sobre la tabla partido mientras se
esta actualizando la misma
Solucion:
	Crear una tabla temporaria de uso automatico por parte
de esta base de datos en donde ir almacenando las fechas y los
equipos de cada partido.

Formato Gral. de este script:

<< SETEOS DE AMBIENTE/PARAMETROS REQUERIDOS PARA LA EJECUCION >>

<< BORRO TABLAS >>

<< CREO TABLAS/CLAVES >>

<< MUESTRO ESTRUCTURA GRAL. DE TABLAS >>

<< IMPLEMENTO CONTROLES DEL SISTEMA >>

En Mundial2.lst queda la salida de este script.

*/

/* Sets para la sesion actual */
clear buffer
set serveroutput on
set verify off
set termout on
ALTER SESSION SET NLS_DATE_FORMAT = 'dd/mm/yyyy'
/

/* Ingreso de parametros */
accept p_len_descs number prompt 'Largo Maximo de las Descripciones:'
/

/* Borro estructuras de datos previas */
DROP TRIGGER trg_iupartido
/
DROP TRIGGER trg_bipartido
/
DROP TRIGGER trg_aipartido
/
DROP TRIGGER trg_adpartido
/
DROP TRIGGER trg_bupartido
/
DROP TRIGGER trg_aupartido
/
DROP TABLE PAR_TMP
/

clear buffer
COMMIT
/

/* Creo estructuras */

/* 
Creo Tabla Partido Temporaria

Partido depende de tabla autoridad, equipo
NOMBRE_EQ1 es not null porque es una relacion obligatoria entre Partido
y Equipo. Idem para NOMBRE_EQ2.
Todo partido involucra a dos equipos. No vuelvo a realizar las
validaciones de nombre de equipo porque ya lo realiza la propia tabla
equipo 
*/
CREATE TABLE PAR_TMP (
   FECHA DATE DEFAULT SYSDATE NOT NULL,
   NOMBRE_EQ1 VARCHAR2(&p_len_descs) NOT NULL,
   NOMBRE_EQ2 VARCHAR2(&p_len_descs) NOT NULL,
   CONSTRAINT pk_par_tmp PRIMARY KEY (FECHA,NOMBRE_EQ1)
)
/

clear buffer

/* Muestro estructuras creadas */
DESC PAR_TMP
/

/*
 Controlar que FECHA-EQ2 no exista en el indice FECHA-EQ1 y FECHA-EQ1  no
 exista en indice FECHA-EQ2

Solucion: Se creo una tabla temporaria PAR_TMP que guardara los mismos datos
que PARTIDO (solo los datos que intervienen en este control). Esta tabla sera
mantenida automaticamente por el sistema.  Cada vez que se produzca un alta,
baja,modificacion sobre PARTIDO tambien se hara sobre PAR_TMP. Esto nos permi
tira hacer el control sobre PAR_TMP en vez de PARTIDO y evitar asi, el lockeo
que realiza Oracle sobre la tabla PARTIDO mientras que sta esta siendo actua-
lizada.

*/
CREATE OR REPLACE TRIGGER trg_bipartido BEFORE INSERT
ON PARTIDO
FOR EACH ROW
DECLARE
   n_cantidad   NUMBER;
BEGIN
   -- Verifico si los nombres de los equipos existen en la temporaria
   DBMS_OUTPUT.PUT_LINE('trg_bipartido ' || to_char(:new.fecha) || ' ' || :new.nombre_eq1);
   SELECT COUNT(*) INTO n_cantidad FROM PAR_TMP PT 
   WHERE PT.FECHA = :new.fecha AND (PT.NOMBRE_EQ1 IN(:new.nombre_eq1,:new.nombre_eq2) OR
                                    PT.NOMBRE_EQ2 IN(:new.nombre_eq1,:new.nombre_eq2));
   IF n_cantidad > 0 THEN
      raise_application_error(-20500,' El dia ' || to_char(:new.fecha) || ' existe un partido ' ||
         ' relacionado con ' || :new.nombre_eq1 || ' o ' || :new.nombre_eq2);
   END IF;
END;
/

CREATE OR REPLACE TRIGGER trg_aipartido AFTER INSERT
ON PARTIDO
FOR EACH ROW
BEGIN
   DBMS_OUTPUT.PUT_LINE('trg_aipartido ' || to_char(:new.fecha) || ' ' || :new.nombre_eq1);
   -- Actualizo tabla temporaria
   INSERT INTO PAR_TMP VALUES(:new.fecha,:new.nombre_eq1,:new.nombre_eq2);
END;
/

CREATE OR REPLACE TRIGGER trg_adpartido AFTER DELETE
ON PARTIDO
FOR EACH ROW
BEGIN
   DBMS_OUTPUT.PUT_LINE('trg_adpartido');
   -- Actualizo tabla temporaria
   DELETE FROM PAR_TMP PT 
   WHERE PT.FECHA = :old.fecha AND PT.NOMBRE_EQ1 = :old.nombre_eq1;
END;
/

CREATE OR REPLACE TRIGGER trg_bupartido BEFORE UPDATE
ON PARTIDO
FOR EACH ROW
DECLARE
   n_cantidad   NUMBER;
BEGIN
   DBMS_OUTPUT.PUT_LINE('trg_bupartido');
   IF UPDATING('FECHA') OR UPDATING('NOMBRE_EQ1') THEN
      -- No se puede modificar ni la fecha ni el primer equipo
      raise_application_error(-20550,' No se puede cambiar la fecha ni '|| 
           ' el nombre del primer equipo.');
   END IF;
   IF UPDATING('NOMBRE_EQ2') THEN 
      -- Verifico si los nombres de los equipos existen en la temporaria
      SELECT COUNT(*) INTO n_cantidad FROM PAR_TMP PT 
      WHERE PT.FECHA = :new.fecha AND (PT.NOMBRE_EQ1 = :new.nombre_eq2 OR
                                       PT.NOMBRE_EQ2 = :new.nombre_eq2);
      IF n_cantidad > 0 THEN
         raise_application_error(-20600,' El dia ' || to_char(:new.fecha) || ' existe un partido ' ||
            ' relacionado con ' || :new.nombre_eq1 || ' o ' || :new.nombre_eq2);
      END IF;
   END IF;
END;
/

CREATE OR REPLACE TRIGGER trg_aupartido AFTER UPDATE
ON PARTIDO
FOR EACH ROW
BEGIN
   DBMS_OUTPUT.PUT_LINE('trg_aupartido');
   IF UPDATING('NOMBRE_EQ2') THEN 
      -- Actualizo tabla temporaria
      UPDATE PAR_TMP SET NOMBRE_EQ2 = :new.nombre_eq2 
      WHERE FECHA = :old.fecha AND NOMBRE_EQ1 = :old.nombre_eq1;
   END IF;
END;
/

COMMIT
/

/*
Actualizo datos en PAR_TMP para ajustar la tabla a los datos
actuales dentro de la tabla partido
*/
INSERT INTO PAR_TMP SELECT FECHA,NOMBRE_EQ1,NOMBRE_EQ2 FROM PARTIDO
/

COMMIT
/

clear buffer
spool off
/*************** fin *****************************************************/